@@ -1,4 +1,5 @@
 
+# $Id: Error.pm,v 1.12 2010/12/02 23:45:57 Martin Exp $
 
 =head1 NAME
 
@@ -6,8 +7,6 @@ WWW::Search::Null::Error - class for testing WWW::Search clients
 
 =head1 SYNOPSIS
 
-=begin example
-
   require WWW::Search;
   my $oSearch = new WWW::Search('Null::Error');
   $oSearch->native_query('Makes no difference what you search for...');
@@ -15,11 +14,6 @@ WWW::Search::Null::Error - class for testing WWW::Search clients
   my $oResponse = $oSearch->response;
   # You get an HTTP::Response object with a code of 500
 
-=end example
-
-=for example_testing
-is($oResponse->code, 500, 'did not get a 500 HTTP::Response');
-
 =head1 DESCRIPTION
 
 This class is a specialization of WWW::Search that only returns an
@@ -30,24 +24,27 @@ actually being connected to any particular search engine.
 
 =head1 AUTHOR
 
-Martin Thurn <mthurn@cpan.org>
+Martin 'Kingpin' Thurn, C<mthurn at cpan.org>, L<http://tinyurl.com/nn67z>.
 
 =cut
 
 package WWW::Search::Null::Error;
 
 use strict;
+use warnings;
 
-use vars qw( @ISA );
-@ISA = qw( WWW::Search );
+use base 'WWW::Search';
+our
+$VERSION = do { my @r = (q$Revision: 1.12 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
+our $MAINTAINER = q{Martin Thurn <mthurn@cpan.org>};
 
-sub native_setup_search
+sub _native_setup_search
   {
   my($self, $native_query, $native_opt) = @_;
   } # native_setup_search
 
 
-sub native_retrieve_some
+sub _native_retrieve_some
   {
   my $self = shift;
   my $response = new HTTP::Response(500,
@@ -58,3 +55,6 @@ sub native_retrieve_some
 
 
 1;
+
+__END__
+
