@@ -1,3 +1,4 @@
+# $rcs = ' $Id: Test.pm,v 2.290 2013/06/23 16:56:39 martin Exp $ ' ;
 
 =head1 NAME
 
@@ -19,42 +20,89 @@ See file test.pl in the WWW-Search-HotBot distribution for a detailed
 
 package WWW::Search::Test;
 
-use Carp;
-use File::Spec;
-
 use strict;
+use warnings;
+
+use Bit::Vector;
+use Carp;
+use Config;
+use Cwd;
+use Data::Dumper;  # for debugging only
+use Date::Manip;
+Date_Init('TZ=-0500');
+use base 'Exporter';
+use File::Path;
+use File::Slurp;
+use File::Spec::Functions qw( :ALL );
+use Test::More;
+use WWW::Search;
 
 use vars qw( $MODE_DUMMY $MODE_INTERNAL $MODE_EXTERNAL $MODE_UPDATE );
 use vars qw( $TEST_DUMMY $TEST_EXACTLY $TEST_BY_COUNTING $TEST_GREATER_THAN $TEST_RANGE );
 use vars qw( $iTest $oSearch $sEngine );
+# If set, will be used as a filename to save HTML when a test fails:
+use vars qw( $sSaveOnError );
 
-require Exporter;
-use vars qw( @EXPORT @EXPORT_OK @ISA );
+use vars qw( @EXPORT );
 @EXPORT = qw( eval_test test
               no_test not_working not_working_with_tests not_working_and_abandoned
               $MODE_DUMMY $MODE_INTERNAL $MODE_EXTERNAL $MODE_UPDATE
               $TEST_DUMMY $TEST_EXACTLY $TEST_BY_COUNTING $TEST_GREATER_THAN $TEST_RANGE
-              new_engine run_test run_gui_test
+              new_engine run_test run_gui_test skip_test count_results
+              tm_new_engine tm_run_test tm_run_test_no_approx
             );
-@EXPORT_OK = qw( );
-@ISA = qw( Exporter );
 
-use Config;
-use Cwd;
-use File::Path;
+use vars qw( $VERSION $bogus_query $websearch );
 
-use vars qw( $VERSION $bogus_query );
-
-$VERSION = '2.14';
+$VERSION = do { my @r = (q$Revision: 2.290 $ =~ /\d+/g); sprintf "%d."."%03d" x $#r, @r };
 $bogus_query = "Bogus" . $$ . "NoSuchWord" . time;
 
 ($MODE_DUMMY, $MODE_INTERNAL, $MODE_EXTERNAL, $MODE_UPDATE) = qw(dummy internal external update);
 ($TEST_DUMMY, $TEST_EXACTLY, $TEST_BY_COUNTING, $TEST_GREATER_THAN, $TEST_RANGE) = (1..10);
 
-# At the time this module is loaded, try to find a working WebSearch:
-my @as = split(/\s/, `WebSearch --VERSION`);
-my $websearch = shift @as;
-undef $websearch unless $websearch =~ m/WebSearch/;
+use constant DEBUG => 0;
+
+=head2 find_websearch
+
+Returns the full path of an executable WebSearch program,
+or undef if none can be found.
+
+=cut
+
+sub find_websearch
+  {
+  unless ($websearch)
+    {
+    # Try to find a working WebSearch:
+    my $sProg = 'WebSearch';
+    my @asTry = ( $sProg );
+    # Try local directory, in case . is not in the path:
+    push @asTry, catfile(curdir, $sProg);
+    push @asTry, catfile(qw( blib script ), $sProg);
+    # See if WebSearch.BAT has been created/installed, and try it with
+    # explicit 'perl' in front:
+    push @asTry, map { ("$_.bat", "$Config{perlpath} $_") } @asTry;
+    DEBUG && print STDERR Dumper(\@asTry);
+ WEBSEARCH_TRY:
+    foreach my $sTry (@asTry)
+      {
+      my $sCmd = "$sTry --VERSION";
+      DEBUG && print STDERR " + W::S::T::find_websearch() cmd ==$sCmd==\n";
+      # Turn off warnings:
+      local $^W = 0;
+      # Wrap it in an eval so we don't die if it fails:
+      my @as = split(/\s/, eval{`$sCmd`});
+      $websearch = shift(@as) || undef;
+      last WEBSEARCH_TRY if $websearch;
+      } # foreach
+    # Prevent undef warning:
+    $websearch ||= '';
+    undef $websearch unless ($websearch =~ m/WebSearch/);
+    # print STDERR "in WWW::Search::Test, websearch is $websearch\n";
+    } # unless
+  return $websearch;
+  } # find_websearch
+
 
 =head2 new
 
@@ -75,7 +123,7 @@ sub new
                 error_count => 0,
                 mode => $MODE_DUMMY,
                 verbose => 0,
-                websearch => $websearch,
+                # websearch => $websearch, # why do we need this?
                }, $class;
   } # new
 
@@ -199,9 +247,7 @@ sub test
     {
     $sExpected = "$low_end..$high_end";
     }
-  
   return if (!$self->relevant_test($sSE));
-  
   print "  trial $file (", $self->{'mode'}, ")\n";
   if (($self->{'mode'} eq $MODE_INTERNAL) && ($query =~ m/$bogus_query/))
     {
@@ -209,9 +255,9 @@ sub test
     return;
     } # if
 
-  my $pwd = File::Spec->curdir();
+  my $pwd = curdir();
   my @asSE = split(/::/, $sSE);
-  my $path = File::Spec->catdir($pwd, 'Test-Pages', @asSE);
+  my $path = catdir($pwd, 'Test-Pages', @asSE);
   mkpath $path;
   if ($self->{'mode'} eq $MODE_UPDATE)
     {
@@ -219,17 +265,17 @@ sub test
     opendir DIR, $path;
     foreach my $afile (readdir DIR)
       {
-      unlink File::Spec->catfile($path, $afile) if $afile =~ m/^$file/;
+      unlink catfile($path, $afile) if $afile =~ m/^$file/;
       } # foreach
     closedir DIR;
     } # if MODE_UPDATE
   # Look at the filename argument we got:
-  my ($v,$d,$f) = File::Spec->splitpath($file);
+  my ($v,$d,$f) = splitpath($file);
   # If it contains no path element (file name only):
   if ($d eq '')
     {
     # Prepend path onto file:
-    $file = File::Spec->catfile($path, $file);
+    $file = catfile($path, $file);
     } # if
   my $o = new WWW::Search($sSE);
   my $version = $o->version;
@@ -242,8 +288,8 @@ sub test
             );
   # --max 209 added by Martin Thurn 1999-09-27.  We never want to
   # fetch more than three pages, if we can at all help it (or do we?)
-  my $websearch = $self->{websearch};
-  $websearch ||= File::Spec->catfile($pwd, 'blib', 'script', 'WebSearch');
+  my $websearch = &find_websearch;
+  $websearch ||= catfile($pwd, 'blib', 'script', 'WebSearch');
   my $cmd = $Config{'perlpath'} . " -MExtUtils::testlib $websearch ";
   $cmd .= $self->{debug} ? '--debug '.$self->{debug} : '';
   $cmd .= " --max 209 --engine $sSE ". $src{$self->{'mode'}} ." -- $query";
@@ -252,7 +298,7 @@ sub test
   open(TRIALFILE, ">$file.trial") || die "$0: cannot open $file.trial for writing ($!)\n";
   open(OUTFILE, ">$file.out") || die "$0: cannot open $file.out for writing ($!)\n" if ($self->{'mode'} eq $MODE_UPDATE);
   my $iActual = 0;
-  while (<TRIALSTREAM>) 
+  while (<TRIALSTREAM>)
     {
     print TRIALFILE $_;
     $iActual++;
@@ -274,10 +320,10 @@ sub test
     return;
     } # if
 
-  if (-f "$file.out") 
+  if (-f "$file.out")
     {
     my ($e, $sMsg) = (0, '');
-    if ($test_method == $TEST_GREATER_THAN) 
+    if ($test_method == $TEST_GREATER_THAN)
       {
       if ($iActual <= $low_end)
         {
@@ -285,7 +331,7 @@ sub test
         $e = 1;
         }
       } # TEST_GREATER_THAN
-    elsif ($test_method == $TEST_RANGE) 
+    elsif ($test_method == $TEST_RANGE)
       {
       $sMsg .= "INTERNAL ERROR, low_end has no value; " unless defined($low_end);
       $sMsg .= "INTERNAL ERROR, high_end has no value; " unless defined($high_end);
@@ -301,11 +347,11 @@ sub test
         $e = 1;
         }
       } # TEST_RANGE
-    elsif ($test_method == $TEST_EXACTLY) 
+    elsif ($test_method == $TEST_EXACTLY)
       {
       $e = &diff("$file.out", "$file.trial") ? 1 : 0;
       } # TEST_EXACTLY
-    elsif ($test_method == $TEST_BY_COUNTING) 
+    elsif ($test_method == $TEST_BY_COUNTING)
       {
       my $iExpected = shift;
       my $iActual = &wc_l("$file.trial");
@@ -321,23 +367,23 @@ sub test
       $sMsg = "INTERNAL ERROR, unknown test method $test_method; ";
       }
 
-    if ($e == 0) 
+    if ($e == 0)
       {
       print "  ok.\n";
       unlink("$file.trial");   # clean up
       } 
-    elsif ($e == 1) 
+    elsif ($e == 1)
       {
       print "DIFFERENCE DETECTED: $query --> $sMsg\n";
       $self->{error_count}++;
       }
-    else 
+    else
       {
       print "INTERNAL ERROR $query --> e is $e.\n";
       $self->{error_count}++;
       }
-    } 
-  else 
+    }
+  else
     {
     print "NO SAVED OUTPUT, can not evaluate test results.\n";
     $self->{error_count}++;
@@ -363,6 +409,7 @@ sub no_test
 NONE
   } # no_test
 
+
 =head2 not_working
 
 Prints a message stating that this backend is known to be broken.
@@ -383,6 +430,7 @@ sub not_working
 BROKEN
   } # not_working
 
+
 =head2 not_working_with_tests
 
 Prints a message stating that this backend is known to be broken
@@ -398,11 +446,12 @@ sub not_working_with_tests
   return if (!$self->relevant_test($engine));
   print <<"KNOWNFAILURE";
   trial none ($engine)
-  Test cases for this search engine are known to fail.  
+  Test cases for this search engine are known to fail.
   You are encouraged to investigate the problem and email its maintainer,
   $maint.
 KNOWNFAILURE
-  } # not_working_with_tests 
+  } # not_working_with_tests
+
 
 =head2 not_working_and_abandoned
 
@@ -412,19 +461,20 @@ Takes two arguments, the backend name and the name of the maintainer.
 
 =cut
 
-sub not_working_and_abandonded
+sub not_working_and_abandoned
   {
   my $self = shift;
   my ($engine, $maint) = @_;
   return if (!$self->relevant_test($engine));
   print <<"ADOPT";
   trial none ($engine)
-  This search engine is known to be non-functional.  
-  You are encouraged to adopt it from its last known maintainer, 
+  This search engine is known to be non-functional.
+  You are encouraged to adopt it from its last known maintainer,
   $maint.
 ADOPT
-  } # not_working_and_abandonded
-        
+  } # not_working_and_abandoned
+
+
 =head2 reset_error_count
 
 Reset the counter of errors to zero.
@@ -438,6 +488,7 @@ sub reset_error_count
   $self->{error_count} = 0;
   } # reset_error_count
 
+
 =head2 wc_l (private, not a method)
 
 Given a filename, count the number of lines of text contained
@@ -460,6 +511,7 @@ sub wc_l
   return $i;
   } # wc_l
 
+
 =head2 diff (private, not a method)
 
 Given two files, returns TRUE if contents are line-by-line
@@ -519,15 +571,32 @@ sub new_engine
   $oSearch = new WWW::Search($sEngine);
   print ref($oSearch) ? '' : 'not ';
   print "ok $iTest\n";
+  $oSearch->env_proxy('yes');
   } # new_engine
 
+=head2 tm_new_engine
+
+Same as new_engine(), but uses Test::More instead of just printing 'ok'.
+
+=cut
+
+sub tm_new_engine
+  {
+  my $sEngine = shift;
+  $oSearch = new WWW::Search($sEngine);
+  Test::More::ok(ref($oSearch), "instantiate WWW::Search::$sEngine object");
+  $oSearch->env_proxy('yes');
+  } # tm_new_engine
+
 =head2 run_test
 
-=head2 run_gui_test
-
-Three arguments: a query string, NOT escaped; a minimum number of expected results; and
+Three arguments: a query string, NOT escaped;
+a minimum number of expected results; and
 a maximum number of expected results.
 Optional fourth argument: integer value to be used as the search_debug.
+Optional fifth argument: send any true value to dump the search results.
+Optional sixth argument: reference to hash of search options (see backend documentation).
+Optional seventh argument: send any true value to NOT escape the query string.
 
 If the minimum is undef, assumes zero.
 If the maximum is undef, does not check.
@@ -538,20 +607,159 @@ Prints 'ok' or 'not ok' and the test number.
 
 sub run_test
   {
-  return &run_our_test('normal', @_);
+  return &_run_our_test('normal', @_);
   } # run_test
 
+=head2 run_gui_test
+
+Same as run_test(), but calls gui_query() instead of native_query().
+
+=cut
+
 sub run_gui_test
   {
-  return &run_our_test('gui', @_);
+  return &_run_our_test('gui', @_);
   } # run_gui_test
 
-sub run_our_test
+
+=head2 tm_run_test
+
+Same as run_test(), but uses Test::More rather than just printing 'ok'.
+
+Note: If you use this function inside a TODO block,
+you must set global variable $TODO rather than a local $TODO,
+and you must set the global $TODO back to empty-string (or undef) at the end of your TODO block.
+For example:
+
+  TODO:
+    {
+    $TODO = 'I have not fixed this yet';
+    tm_run_test(...);
+    $TODO = '';
+    } # end of TODO block
+
+=cut
+
+sub tm_run_test
   {
-  my ($sType, $sQuery, $iMin, $iMax, $iDebug, $iPrintResults) = @_;
+  _tm_run_test(@_, 1);
+  } # tm_run_test
+
+sub _tm_run_test
+  {
+  # Last argument is boolean, whether to check approx_result_count:
+  my $iApprox = pop(@_) || 0;
+  # Remaining args, same as count_results():
+  my ($sType, $sQuery, $iMin, $iMax) = @_;
+  my $iCount = count_results(@_);
+  my $iAnyFailure = 0;
+  $iAnyFailure++ unless Test::More::is($oSearch->response->code, 200, 'got valid HTTP response');
+  if (defined $iMin)
+    {
+    $iAnyFailure++ unless Test::More::cmp_ok($iMin, '<=', $iCount,
+                                             qq{lower-bound num-hits for query=$sQuery});
+    if ($iApprox)
+      {
+      $iAnyFailure++ unless Test::More::cmp_ok($iMin, '<=', $oSearch->approximate_result_count,
+                                               qq{lower-bound approximate_result_count});
+      } # if
+    } # if
+  if (defined $iMax)
+    {
+    $iAnyFailure++ unless Test::More::cmp_ok($iCount, '<=', $iMax,
+                                             qq{upper-bound num-hits for query=$sQuery});
+    if ($iApprox)
+      {
+      $iAnyFailure++ unless Test::More::cmp_ok($oSearch->approximate_result_count, '<=', $iMax,
+                                               qq{upper-bound approximate_result_count});
+      } # if
+    } # if
+  $sSaveOnError ||= q'';
+  if ($iAnyFailure && ($sSaveOnError ne q''))
+    {
+    write_file($sSaveOnError, { err_mode => 'quiet'}, $oSearch->response->content);
+    Test::More::diag(qq'HTML was saved in $sSaveOnError');
+    } # if
+  } # _tm_run_test
+
+=head2 tm_run_test_no_approx
+
+Same as tm_run_test, but does NOT check the approximate_result_count.
+
+=cut
+
+sub tm_run_test_no_approx
+  {
+  &_tm_run_test(@_, 0);
+  } # tm_run_test_no_approx
+
+=head2 count_results
+
+Run a query, and return the actual (not approximate) number of hits.
+Required first argument determines which backend query method to call: 'gui' to call gui_query(), anything else to call native_query().
+Remaining arguments are same as all the run_test() arguments.
+
+=cut
+
+sub count_results
+  {
+  my ($sType, $sQuery, $iMin, $iMax, $iDebug, $iPrintResults, $rh, $iDoNotEscape) = @_;
+  # print STDERR qq{ DDD count_results raw args($sType,$sQuery,$iMin,$iMax,$iDebug,$iPrintResults,$rh,$iDoNotEscape)\n};
   $iDebug ||= 0;
   $iPrintResults ||= 0;
-  carp ' --- min/max values out of order?' if defined($iMin) && defined($iMax) && ($iMax < $iMin);
+  $rh->{'search_debug'} = $iDebug;
+  carp ' --- min/max values out of order?' if (defined($iMin) && defined($iMax) && ($iMax < $iMin));
+  $oSearch->reset_search;
+  $iMin ||= 0;
+  # While $iMax is the number the user wants to compare, $iMaxAbs is
+  # the actual number we apply to the search:
+  my $iMaxAbs;
+  if (! defined($iMax))
+    {
+    # User said upper limit is 'undef'; just make sure we get the
+    # mininum:
+    $iMaxAbs = $iMin + 1;
+    } # if
+  else
+    {
+    # Give a little breathing room, so we'll notice if there are too
+    # many returned:
+    $iMaxAbs = $iMax + 1;
+    }
+  $oSearch->maximum_to_retrieve($iMaxAbs);
+  $iTest++;
+  $sQuery = WWW::Search::escape_query($sQuery) unless $iDoNotEscape;
+  # print STDERR " + in WWW::Search::Test::count_results, iDebug = $iDebug\n";
+  if ($sType eq 'gui')
+    {
+    $oSearch->gui_query($sQuery, $rh);
+    }
+  else
+    {
+    $oSearch->native_query($sQuery, $rh);
+    }
+  $oSearch->login($ENV{WWW_SEARCH_USERNAME}, $ENV{WWW_SEARCH_PASSWORD});
+  my @aoResults = $oSearch->results();
+  if ($iPrintResults)
+    {
+    my $i = 1;
+    foreach my $oResult (@aoResults)
+      {
+      print $i++, '. ', $oResult->url, "\n";
+      foreach my $sField (qw( title description score change_date index_date size company location source ))
+        {
+        print "  $sField==", $oResult->$sField, "==\n" if defined($oResult->$sField);
+        } # foreach
+      } # foreach
+    } # if
+  return scalar(@aoResults);
+  } # count_results
+
+
+sub _run_our_test
+  {
+  my ($sType, $sQuery, $iMin, $iMax, $iDebug, $iPrintResults) = @_;
+  my $iResults = &count_results(@_);
   my $sExpect;
   if (! defined($iMax))
     {
@@ -565,55 +773,121 @@ sub run_our_test
     {
     $sExpect = "$iMin..$iMax";
     }
-  $iMin ||= 0;
-  if ($iMin == $iMax)
-    {
-    # The caller expects an exact result set.  Do not limit how many
-    # results are returned.
-    $oSearch->maximum_to_retrieve(99999);
-    }
-  else
-    {
-    if (defined $iMax)
-      {
-      $oSearch->maximum_to_retrieve($iMax + 1);
-      }
-    else
-      {
-      $oSearch->maximum_to_retrieve($iMin + 1);
-      $iMax = 999999;
-      }
-    }
-  $iTest++;
-  $sQuery = WWW::Search::escape_query($sQuery);
-  # print STDERR " + in WWW::Search::Test::run_our_test, iDebug = $iDebug\n";
-  if ($sType eq 'gui')
-    {
-    $oSearch->gui_query($sQuery,
-                          { 'search_debug' => $iDebug, },
-                       );
-    }
-  else
-    {
-    $oSearch->native_query($sQuery,
-                             { 'search_debug' => $iDebug, },
-                          );
-    }
-  my @aoResults = $oSearch->results();
-  if ($iPrintResults)
-    {
-    $, = "\n";
-    print map { $_->url .' == '. $_->description } @aoResults;
-    print "\n";
-    } # if
-  my $iResults = scalar(@aoResults);
+  $iMax = 999999 unless defined ($iMax);
   if (($iResults < $iMin) || ($iMax < $iResults))
     {
     print STDERR " --- got $iResults results for $sType $sEngine query '$sQuery', but expected $sExpect\n";
     print STDOUT 'not ';
     } # if
   print STDOUT "ok $iTest\n";
-  } # run_test
+  } # _run_our_test
+
+
+=head2 skip_test
+
+You can call this function instead of run_test() or run_gui_test()
+if the current test must be skipped for any reason.
+
+=cut
+
+sub skip_test
+  {
+  $iTest++;
+  print STDOUT "skip $iTest\n";
+  } # skip_test
+
+
+=head2 test_most_results
+
+Given an arrayref of things to test,
+runs all those things against all the results of the most-recently executed test search.
+
+=cut
+
+sub test_most_results
+  {
+  my $rara = shift;
+  my $fPct = shift || 0.80;
+  my $iCount = scalar(@$rara);
+  my $iAnyFailed = my $iResult = 0;
+  my %hioExemplar;
+  my %hiiFailed;
+  # Create a bit vector large enough to hold one bit for each test:
+  my $oV = new Bit::Vector($iCount);
+  # Turn on all the bits (we will turn off bits when tests fail):
+  $oV->Fill;
+  my $iVall = $oV->to_Dec;
+  my $sCodeAll = q{};
+  my $iTest = 0;
+ TEST:
+  foreach my $ra (@$rara)
+    {
+    # print STDERR " DDD ra is ", Dumper($ra);
+    my ($sField, $sCmp, $sValue, $sDesc) = @$ra;
+    my $sCode;
+    if ($sCmp eq 'like')
+      {
+      $sCode = "(\$oResult->$sField =~ m!$sValue!)";
+      } # if
+    elsif ($sCmp eq 'unlike')
+      {
+      $sCode = "(\$oResult->$sField !~ m!$sValue!)";
+      } # if
+    elsif ($sCmp eq 'date')
+      {
+      $sCode = "((ParseDate(\$oResult->$sField) || '') ne q{})";
+      } # if
+    else
+      {
+      $sCode = "(\$oResult->$sField $sCmp $sValue)";
+      }
+    $sCode = <<"ENDCODE";
+if (! $sCode)
+  {
+  \$oV->Bit_Off($iTest);
+  \$hiiFailed{'$sField'}++;
+  } # if
+ENDCODE
+    $sCodeAll .= $sCode;
+    $iTest++;
+    } # foreach TEST
+  $sCodeAll .= "1;\n";
+  # print STDERR " DDD the test is ===$sCodeAll===\n";
+ RESULT:
+  foreach my $oResult ($oSearch->results())
+    {
+    $iResult++;
+    # Turn on all the bits (we will turn off bits when tests fail):
+    $oV->Fill;
+    # print STDERR " DDD eval the test...\n";
+    if (! eval $sCodeAll)
+      {
+      print STDERR $@;
+      } # if
+    # Now look at the value of the Bit::Vector after running the tests:
+    my $iV = $oV->to_Dec;
+    if ($iV < $iVall)
+      {
+      # At least one of the bits got turned off (i.e. a test failed):
+      $hioExemplar{$iV} = $oResult;
+      $iAnyFailed++;
+      } # if
+    } # foreach RESULT
+  ok($iResult, qq{got more than zero results ($iResult, to be exact)});
+  while (my ($sItem, $iFailed) = each %hiiFailed)
+    {
+    my $fPctFailed = ($iFailed / $iResult);
+    ok($fPctFailed < (1 - $fPct), sprintf(qq{%0.1f%% of %s tests failed}, $fPctFailed * 100, $sItem));
+    } # while
+  if ($iAnyFailed)
+    {
+    Test::More::diag(" Here are result(s) that exemplify test failure(s):");
+    foreach my $oResult (values %hioExemplar)
+      {
+      Test::More::diag(Dumper($oResult));
+      } # while
+    } # if
+  } # test_most_results
 
 1;
 
